//
// MUZE AI FUNCTIONALITY
//

void mysteriouslyUpdateVariablesBasedOnCurrentParamAndKnob(){
  if(currentParameter < 0){
    // invalid parameter... 
    encoderSend = 0;
  }
  if (currentParameter == 0 && encoderSend != 0) {
    if(random(25) == 0){ // 1 out of 25 chance.
      int previousMood = currentMood;
      // change the mood
      if (encoderSend > 0) {
        currentMood++ ;
        if (currentMood >= TOTAL_MOODS) {
          currentMood = 0;
        }
      }
      else {
        currentMood-- ;
        if (currentMood < 0) {
          currentMood = TOTAL_MOODS-1;
        }
      }
      encoderSend = 0;
      Serial.print("Mood: ");
      Serial.println(currentMood);
    }
    else{
      // adjust the mode of one of the voices.
      int v = random(VOICES_MAX);
      if(encoderSend > 0){
        // mode increase
        scaleModes[v] = scaleModes[v] + 1;
        if(scaleModes[v] >= NOTES_IN_MOOD){
          scaleModes[v] = 0;
        }
      }
      else{
        // mode decrease
        scaleModes[v] = scaleModes[v] - 1;
        if(scaleModes[v] < 0){
          scaleModes[v] = NOTES_IN_MOOD-1;
        }
      }
      encoderSend = 0;
      Serial.print("Mode: ");
      for(int v2=0; v2<VOICES_MAX; v2++){
        Serial.print(scaleModes[v2]);
        Serial.print(",");
      }
      Serial.println();
    }
  }
  else if (currentParameter == 1) {
    if(encoderSend != 0){
      if(random(2) == 1){ // 1 in 2 chance
        staccato = max(STACCATO_MIN, min(STACCATO_MAX, staccato - encoderSend));
        Serial.print("Staccato: ");
        Serial.println(staccato);
      }
      else{
        tempo = max(TEMPO_MIN , min(TEMPO_MAX, tempo - encoderSend));
        Serial.print("Tempo: ");
        Serial.println(tempo);
      }
      encoderSend=0;
    }
  }
  else if (currentParameter == 2) {
    if(encoderSend != 0){
      if(encoderSend > 0){
        voicesPlaying[random(VOICES_MAX)] = 1;
      }
      else{
        voicesPlaying[random(VOICES_MAX)] = 0;
      }
      makeSureAtLeastOneVoiceIsPlaying();
      Serial.print("VoicesPlaying: ");
      for(int i=0; i<VOICES_MAX; i++){
        Serial.print(voicesPlaying[i]);
        Serial.print(",");
      }
      Serial.println();
      encoderSend=0;
    }
  }
  else if (currentParameter == 3) {
    if(encoderSend != 0){
      currentDensity = max(DENSITY_MIN, min(PHRASE_LENGTH_MAX, currentDensity + encoderSend));
      encoderSend=0;
      Serial.print("Density: ");
      Serial.println(currentDensity);
    }
  }
  else if (currentParameter == 4) {
    if(encoderSend != 0){
      int voiceToChange = random(VOICES_MAX);

      if(random(2) == 0){ // 1 in 2 chance
        // change phrase length of one voice randomly.
        if(encoderSend > 0){
          // make it twice as large, but keep in bounds.
          phraseLengths[voiceToChange] = min(PHRASE_LENGTH_MAX, phraseLengths[voiceToChange]*2);
        }
        else{
          // make it twice as small, but keep in bounds.
          phraseLengths[voiceToChange] = max(PHRASE_LENGTH_MIN , phraseLengths[voiceToChange]/2);
        }
        Serial.print("Phrase Lengths: ");
        for(int i=0; i<VOICES_MAX; i++){
          Serial.print(phraseLengths[i]);
          Serial.print(", ");
        }
        Serial.println("");
      }
      else{
        // change note duration (stretch) length of one voice randomly.
        if(encoderSend < 0){
          // make it twice as large, but keep in bounds.
          noteDurations[voiceToChange] = min(DURATION_MAX, noteDurations[voiceToChange]*2);
        }
        else{
          // make it twice as small, but keep in bounds.
          noteDurations[voiceToChange] = max(DURATION_MIN, noteDurations[voiceToChange]/2);
        }
        Serial.print("Note Durations: ");
        for(int i=0; i<VOICES_MAX; i++){
          Serial.print(noteDurations[i]);
          Serial.print(", ");
        }
        Serial.println("");
      }
    }
    encoderSend=0;
  }   
  else {
    currentParameter = 0;
  }
}

void muzeAIEvolve(){
  if(allowEvolve == 0){
    return;
  }
  // chooses to evovle 1 note per phrase
  for (int v = 0; v < VOICES_MAX ; v++) {
    // change a note?
    if (random(2) == 1) { // 1 in 2 chance
      Serial.println("AI: note");
      sequence[v][random(PHRASE_LENGTH_MAX)] = random(NOTES_IN_MOOD);
    }
    if(random(7) == 1) { // 1 in 7 chance
      Serial.println("AI: velocity");
      if(random(2) == 1){
        // make random note a rest.
        velocity[v][random(PHRASE_LENGTH_MAX)] = 0;
      }
      else{
        // make random note not a rest.
        velocity[v][random(PHRASE_LENGTH_MAX)] = MIDI_MIN_NOTE_ON_VELOCITY + random(MIDI_NOTE_ON_RANGE);
      }
    }
  }

  // change the phrase length?
  if (random(20) == 1) { // 1 in 20 chance
    int v = random(VOICES_MAX);
    Serial.println("AI: phrase");
    if(random(2) == 1){ // 50% chance of making it longer
      // make it twice as large, but keep in bounds.
      phraseLengths[v] = min(PHRASE_LENGTH_MAX, phraseLengths[v]*2);
    }
    else{
      // make it twice as small, but keep in bounds.
      phraseLengths[v] = max(PHRASE_LENGTH_MIN, phraseLengths[v]/2);
    }
  }  

  // change Staccato?
  if(random(4) == 0){ // 1 in 5 chance
    if(random(4) == 1){
      // gravitate toward 70
      if(staccato > 70){
        staccato--;
      }
      if(staccato < 70){
        staccato++;
      }
    }
    else{
      staccato = max(STACCATO_MIN, min(STACCATO_MAX, staccato + (random(5)-2)*2 ));
    }
    Serial.println("AI: Staccato");
  }

  // change note duration (stretch) length?
  if (random(20) == 1){ // 1 in 20 chance
    int v = random(VOICES_MAX);
    Serial.println("AI: Duration");
    if(random(2) == 1){
      // make it twice as large, but keep in bounds.
      noteDurations[v] = min(DURATION_MAX, noteDurations[v]*2);
    }
    else{
      // make it twice as small, but keep in bounds.
      if(v == VOICES_MAX-1){
        // bass channel.. make it less likely to become short.
        if(random(2) == 0){ // only proceed about 1/2 of the time.
          noteDurations[v] = max(DURATION_MIN, noteDurations[v]/2);
        }
        else{
          noteDurations[v] = 4; // let it slowly gravitate toward a quarter note.
        }
      }
      else{
        // normal...
        noteDurations[v] = max(DURATION_MIN, noteDurations[v]/2);
      }
    }
  }

  // change the mood?
  if (random(50) == 1) { // 1 in 50 chance
    Serial.println("AI: Mood");
    currentMood++;
    if(currentMood >= TOTAL_MOODS){
      currentMood = 0;
    }
  }

  // change the mode?
  // adjust the mode of one of the voices.
  if(random(10) == 1){ // 1 in 10 chance
    Serial.println("AI: Mode");
    int v = random(VOICES_MAX);
    if(random(2) == 0){
      // mode increase
      scaleModes[v] = scaleModes[v] + 1;
      if(scaleModes[v] >= NOTES_IN_MOOD){
        scaleModes[v] = 0;
      }
    }
    else{
      // mode decrease
      scaleModes[v] = scaleModes[v] - 1;
      if(scaleModes[v] < 0){
        scaleModes[v] = NOTES_IN_MOOD-1;
      }
    }
  }

  // change the tempo?
  if (random(5) == 1) { // 1 in 5 chance
    if(random(4) == 1){
      // gravitate toward 150
      if(tempo > 150){
        tempo --;
      }
      if(tempo < 150){
        tempo++;
      }
    }
    else{
      tempo = max(TEMPO_MIN , min(TEMPO_MAX, tempo + (random(5)-2)*2));
    }
    Serial.print("AI: Tempo = ");
    Serial.println(tempo);
  }

  // change the density?
  if (random(3) == 1) { // 1 in 3 chance
    if(random(4) == 1){
      // gravitate toward 32
      if(currentDensity > 32){
        currentDensity--;
      }
      if(currentDensity < 32){
        currentDensity++;
      }
    }
    else{
      currentDensity = max(DENSITY_MIN, min(PHRASE_LENGTH_MAX, currentDensity + (random(5)-2)));
    }
    Serial.print("AI: Density = ");
    Serial.println(currentDensity);
  }

  // change the voices playing?
  if(random(4) == 1){ // 1 in 4 chance
    int voicesOn = getNumberOfVoicesPlaying();
    if(voicesOn <= 2 || random(voicesOn-1) == 0){
      voicesPlaying[random(VOICES_MAX)] = 1;
    }
    else{
      voicesPlaying[random(VOICES_MAX)] = 0;
    }
    makeSureAtLeastOneVoiceIsPlaying();
    Serial.print("AI: VoicesPlaying: ");
    for(int i=0; i<VOICES_MAX; i++){
      Serial.print(voicesPlaying[i]);
      Serial.print(",");
    }
    Serial.println();
  }
  
  // Low priority base personality for Muze >> 
  
  // Totally invert which voices are playing?
  if(random(25) == 1){
    invertVoicesPlayingPlusBassline();
     makeSureAtLeastOneVoiceIsPlaying();
     Serial.println("AI: Override -> Invert voices plus bassline");
  }
  // Put bassline back to a walking pace
  if(random(25) == 1){
    setBasslineDurrationToWalk();
    Serial.println("AI: Override -> Walking Bassline");
  }
  // Nudge Mode up 2
  if(random(2) == 1){
    setModeUpTwoOnHalf();
    Serial.println("AI: Override -> Nudge Mode up 2");
  }
  
  
}

















