//
// MUZE EEPROM FUNCTIONALITY
//
#include <EEPROM.h>

// Things to save....
// Large Arrays  :  sequence, velocity
// Smaller Arrays:  phraseLengths, noteDurations, rootNotes, scaleModes, voicesPlaying
// Variables     :  currentMood, currentDensity, tempo, staccato 

#define TOTAL_ARRAYS_TO_SAVE 3
#define SAVE_DIV             8 // break up part into sections of this length
#define MAX_SAVE_STATES      VOICES_MAX * TOTAL_ARRAYS_TO_SAVE * SAVE_DIV

int saveStateCounter = 0;

#define EEPOffset_Mood         0
#define EEPOffset_Density      1
#define EEPOffset_Tempo        2
#define EEPOffset_Staccato     3

#define EEPOffset_AllowEvolve  4
#define EEPOffset_DataVersion 60

#define EEPOffset_Array_Sequence       64
#define EEPOffset_Array_Velocity       EEPOffset_Array_Sequence      + VOICES_MAX*PHRASE_LENGTH_MAX  // 2*4*64 = 512bytes = 0.5k (1/8 of our total EEPROM).
#define EEPOffset_Array_PhraseLengths  EEPOffset_Array_Velocity      + VOICES_MAX*PHRASE_LENGTH_MAX  // 2*4*64 = 512bytes = 0.5k (1/8 of our total EEPROM).
#define EEPOffset_Array_NoteDurations  EEPOffset_Array_PhraseLengths + VOICES_MAX
#define EEPOffset_Array_RootNotes      EEPOffset_Array_NoteDurations + VOICES_MAX
#define EEPOffset_Array_ScaleModes     EEPOffset_Array_RootNotes     + VOICES_MAX
#define EEPOffset_Array_VoicesPlaying  EEPOffset_Array_ScaleModes    + VOICES_MAX


long startSaveTime = 0L;
int eepromVal = 0;
int saveStateStep(){
  startSaveTime = millis();
  if(saveStateCounter >=  MAX_SAVE_STATES){
    eepromVal = EEPROMReadInt(EEPOffset_Mood);
    if(eepromVal != currentMood){
      Serial.print("EEPROM: Mood -> ");
      Serial.println(currentMood);
      EEPROMWriteInt(EEPOffset_Mood, currentMood);
    }
    eepromVal = EEPROMReadInt(EEPOffset_Density);
    if(eepromVal != currentDensity){
      Serial.print("EEPROM: Density -> ");
      Serial.println(currentDensity);
      EEPROMWriteInt(EEPOffset_Density, currentDensity);
    }
    eepromVal = EEPROMReadInt(EEPOffset_Tempo);
    if(eepromVal != tempo){
      Serial.print("EEPROM: Tempo -> ");
      Serial.println(tempo);
      EEPROMWriteInt(EEPOffset_Tempo, tempo);
    }
    eepromVal = EEPROMReadInt(EEPOffset_Staccato);
    if(eepromVal != staccato){
      Serial.print("EEPROM: Staccato -> ");
      Serial.println(staccato);
      EEPROMWriteInt(EEPOffset_Staccato, staccato);
    }
    eepromVal = EEPROMReadInt(EEPOffset_AllowEvolve);
    if(eepromVal != allowEvolve){
      Serial.print("EEPROM: AllowEvolve -> ");
      Serial.println(allowEvolve);
      EEPROMWriteInt(EEPOffset_AllowEvolve, allowEvolve);
    }
    eepromVal = EEPROMReadInt(EEPOffset_DataVersion);
    if(eepromVal != MUZE_VERSION){
      // we're at the end of a full save cycle. Indicate the version.
      EEPROMWriteInt(EEPOffset_DataVersion, MUZE_VERSION);
      Serial.print("EEPROM: Initial save complete. DataVersion is now >>> ");
      Serial.println(MUZE_VERSION);
    }
    saveStateCounter = 0;
  }
  else{
    int v = (saveStateCounter/SAVE_DIV) % VOICES_MAX;
    int arrayNum = (saveStateCounter/SAVE_DIV) / VOICES_MAX;
    int divIndex = saveStateCounter % SAVE_DIV;
    // TODO: save the arrays!
    switch(arrayNum){
    case 0: 
      {
        //Sequence
        for(int i=divIndex*SAVE_DIV; i<((divIndex+1)*SAVE_DIV); i++){
          eepromVal = EEPROMReadInt(EEPOffset_Array_Sequence + v*PHRASE_LENGTH_MAX + i);
          if(eepromVal != sequence[v][i]){
            Serial.print("EEPROM: v[");
            Serial.print(v);
            Serial.print("], n[");
            Serial.print(i);
            Serial.print("] -> note:");
            Serial.println(sequence[v][i]);
            EEPROMWriteInt(EEPOffset_Array_Sequence + v*PHRASE_LENGTH_MAX + i, max(0, min(NOTES_IN_MOOD-1, sequence[v][i])));
          }
        }
        break;
      }
    case 1: 
      {
        //Velocity
        for(int i=divIndex*SAVE_DIV; i<((divIndex+1)*SAVE_DIV); i++){
          eepromVal = EEPROMReadInt(EEPOffset_Array_Velocity + v*PHRASE_LENGTH_MAX + i);
          if(eepromVal != velocity[v][i]){
            Serial.print("EEPROM: v[");
            Serial.print(v);
            Serial.print("], n[");
            Serial.print(i);
            Serial.print("] -> vel:");
            Serial.println(velocity[v][i]);
            EEPROMWriteInt(EEPOffset_Array_Velocity + v*PHRASE_LENGTH_MAX + i, velocity[v][i]);
          }
        }
        break;
      }
    case 2: 
      {
        //phraseLengths
        if(divIndex == 0){
          eepromVal = EEPROMReadInt(EEPOffset_Array_PhraseLengths + v);
          if(eepromVal != phraseLengths[v]){
            Serial.print("EEPROM: phraseLengths[");
            Serial.print(v);
            Serial.print("] -> ");
            Serial.println(phraseLengths[v]);
            EEPROMWriteInt(EEPOffset_Array_PhraseLengths + v, phraseLengths[v]);
          }
        }
        //noteDurations
        if(divIndex == 1){
          eepromVal = EEPROMReadInt(EEPOffset_Array_NoteDurations + v);
          if(eepromVal != noteDurations[v]){
            Serial.print("EEPROM: noteDurations[");
            Serial.print(v);
            Serial.print("] -> ");
            Serial.println(noteDurations[v]);
            EEPROMWriteInt(EEPOffset_Array_NoteDurations + v, noteDurations[v]);
          }
        }
        //rootNotes
        if(divIndex == 2){
          eepromVal = EEPROMReadInt(EEPOffset_Array_RootNotes + v);
          if(eepromVal != rootNotes[v]){
            Serial.print("EEPROM: rootNotes[");
            Serial.print(v);
            Serial.print("] -> ");
            Serial.println(rootNotes[v]);
            EEPROMWriteInt(EEPOffset_Array_RootNotes + v, rootNotes[v]);
          }
        }
        //scaleModes
        if(divIndex == 3){
          eepromVal = EEPROMReadInt(EEPOffset_Array_ScaleModes + v);
          if(eepromVal != scaleModes[v]){
            Serial.print("EEPROM: scaleModes[");
            Serial.print(v);
            Serial.print("] -> ");
            Serial.println(scaleModes[v]);
            EEPROMWriteInt(EEPOffset_Array_ScaleModes + v, scaleModes[v]);
          }
        }
        //voicesPlaying
        if(divIndex == 4){
          eepromVal = EEPROMReadInt(EEPOffset_Array_VoicesPlaying + v);
          if(eepromVal != voicesPlaying[v]){
            Serial.print("EEPROM: voicesPlaying[");
            Serial.print(v);
            Serial.print("] -> ");
            Serial.println(voicesPlaying[v]);
            EEPROMWriteInt(EEPOffset_Array_VoicesPlaying + v, voicesPlaying[v]);
          }
        }
        break;
      }
    default: 
      {
        Serial.println("default case of EEPROM?");
        break;
      }
    }
    saveStateCounter++;
  }  
  return millis()-startSaveTime;
}


int loadSavedState(){
  startSaveTime = millis();
  Serial.println("Loading saved state...");
  int ver = EEPROMReadInt(EEPOffset_DataVersion);
  if(!(ver == MUZE_VERSION)){
    Serial.print("Aborting load. Looking for version ");
    Serial.print(MUZE_VERSION);
    Serial.print(" but found ");
    Serial.println(ver);
    loadInitialRandomSequence();
    return 0;
  }
  // looking good.  go ahead and load all the variables.
  currentMood    = EEPROMReadInt(EEPOffset_Mood);
  currentDensity = EEPROMReadInt(EEPOffset_Density);
  tempo          = EEPROMReadInt(EEPOffset_Tempo);
  staccato       = EEPROMReadInt(EEPOffset_Staccato);
  allowEvolve     = EEPROMReadInt(EEPOffset_AllowEvolve);
  // sequence...
  for(int v=0; v<VOICES_MAX; v++){
    for(int p=0; p<PHRASE_LENGTH_MAX; p++){
      sequence[v][p] = EEPROMReadInt(EEPOffset_Array_Sequence + v*PHRASE_LENGTH_MAX + p);
    }
  }
  // velocity...
  for(int v=0; v<VOICES_MAX; v++){
    for(int p=0; p<PHRASE_LENGTH_MAX; p++){
      velocity[v][p] = EEPROMReadInt(EEPOffset_Array_Velocity + v*PHRASE_LENGTH_MAX + p);
    }
  }
  // phraseLengths
  for(int v=0; v<VOICES_MAX; v++){
    phraseLengths[v] = EEPROMReadInt(EEPOffset_Array_PhraseLengths + v);
  }
  // noteDurations
  for(int v=0; v<VOICES_MAX; v++){
    noteDurations[v] = EEPROMReadInt(EEPOffset_Array_NoteDurations + v);
  }
  // rootNotes
  for(int v=0; v<VOICES_MAX; v++){
    rootNotes[v] = EEPROMReadInt(EEPOffset_Array_RootNotes + v);
  }
  // scaleModes
  for(int v=0; v<VOICES_MAX; v++){
    scaleModes[v] = EEPROMReadInt(EEPOffset_Array_ScaleModes + v);
  }
  // voicesPlaying
  for(int v=0; v<VOICES_MAX; v++){
    voicesPlaying[v] = EEPROMReadInt(EEPOffset_Array_VoicesPlaying + v);
  }

  return millis()-startSaveTime;
}

void EEPROMWriteInt(int p_address, int p_value){
  byte lowByte  = ((p_value >> 0) & 0x00FF);
  byte highByte = ((p_value >> 8) & 0x00FF);
  EEPROM.write(p_address*2,   lowByte );
  EEPROM.write(p_address*2+1, highByte);
}
int EEPROMReadInt(int p_address){
  byte lowByte  = EEPROM.read(p_address*2);
  byte highByte = EEPROM.read(p_address*2+1);
  return ((lowByte << 0) & 0x00FF) + ((highByte << 8) & 0xFF00);
}




