//
// MUZE MIDI FUNCTIONALITY
//

#define MIDI_PROGRAM_MUZE_VOICE_A   80  //  80 = square lead is the norm; 46 = harp
#define MIDI_PROGRAM_MUZE_VOICE_B  115  // 115 = woodblock
#define MIDI_PROGRAM_MUZE_VOICE_C   32  //  32 = acoustic bass
#define MIDI_PROGRAM_MUZE_VOICE_D   53  //  53 = voice oohs
// ############################################################################################
// High-Level MIDI Functions
// ############################################################################################
void muzePlayNote(int note, int velocity){
  midiNoteOn(1, note, velocity);
  //midiNoteOn(2, note, velocity);
  midiNoteOn(3, note, velocity);
  //midiNoteOn(4, note, velocity);
  digitalWrite(13, HIGH);
}
void muzeStopNote(int note){
  midiNoteOff(1, note, 0);
  //midiNoteOff(2, note, 0);
  midiNoteOff(3, note, 0);
  //midiNoteOff(4, note, 0);
  digitalWrite(13, LOW);
}

int getMIDINoteAtPosition(int voice, int position){
  int notePtr = sequence[voice][position];
  int notePtrWithMode = (notePtr + scaleModes[voice] + NOTES_IN_MOOD) % NOTES_IN_MOOD;
  int midiNoteToPlay = rootNotes[voice] + mood[notePtrWithMode];
  return midiNoteToPlay;
}

// ############################################################################################
// Mid-Level MIDI Functions
// ############################################################################################
void initMIDI(){
  Serial2.begin(31250);
  midiSetProgram(1, MIDI_PROGRAM_MUZE_VOICE_A);
  midiSetVolume(1, 60);
  midiAllNotesOff(1);
  midiSetProgram(2, MIDI_PROGRAM_MUZE_VOICE_B);
  midiSetVolume(2, 80);
  midiAllNotesOff(2);
  midiSetProgram(3, MIDI_PROGRAM_MUZE_VOICE_C);
  midiSetVolume(3, 95);
  midiAllNotesOff(3);
  midiSetProgram(4, MIDI_PROGRAM_MUZE_VOICE_D);
  midiSetVolume(4, 90);
  midiAllNotesOff(4);
}
void midiNoteOff(int channel, int note, int velocity){
  if(channel >= 16 || note > 127 || velocity > 127){
    Serial.print("midiNoteOff: Data out of range. chan:");
    Serial.print(channel);
    Serial.print(", note:");
    Serial.print(note);
    Serial.print(", vel:");
    Serial.println(velocity);
  }
  else{
    // looks good, send it!
    midiSendCommand3(0x80 | channel, note, velocity);
  }
}
void midiAllNotesOff(int channel){
  int i = 0;
  for(i=0; i<128; i++){
    midiNoteOff(channel, i, 0);
  }
}
void midiNoteOn(int channel, int note, int velocity){
  if(channel >= 16 || note > 127 || velocity > 127){
    Serial.println("midiNoteOn: Data out of range.");
  }
  else{
    // looks good, send it!
    midiSendCommand3(0x90 | channel, note, velocity);
  }
}
void midiSetProgram(int channel, int program){
  if(channel >= 16 || program > 127){
    Serial.println("midiSetProgram: Data out of range.");
  }
  else{
    // looks good, send it!
    midiSendCommand2(0xC0 | channel, program);
  }
}
void midiSetVolume(int channel, int volume){
  if(channel >= 16 || volume > 127){
    Serial.println("midiSetVolume: Data out of range.");
  }
  else{
    // looks good, send it!
    midiSendCommand3(0xB0 | channel, 7, volume);
  }
}
void midiSendCommand3(int cmd, int A, int B) {
  // check that data is in range.
  if(cmd <= 127 || A > 127 || B > 127){
    Serial.println("midiSendCommand3: Data out of range.");
  }
  else{
    // looks good.  send it!
    Serial2.print(cmd, BYTE);
    Serial2.print(A, BYTE);
    Serial2.print(B, BYTE);
  }
}
void midiSendCommand2(int cmd, int A) {
  // check that data is in range.
  if(cmd <= 127 || A > 127){
    Serial.println("midiSendCommand2: Data out of range.");
  }
  else{
    // looks good.  send it!
    Serial2.print(cmd, BYTE);
    Serial2.print(A, BYTE);
  }
}
void checkForMIDIData(){
  if(Serial2.available()){
    Serial.print("Serial Data In: ");
    while(Serial2.available()){
      Serial.print(" 0x");
      Serial.print(Serial2.read(), HEX);
    }
    Serial.println("");
  }
}


