//
// MUZE MUSIC MAKING FUNCTIONALITY
//

void loadInitialRandomSequence(){
  // randomize initial tune -- ideally we swap this out with EEPROM reads on startup of the last playing state.
  Serial.print("Populating initial sequence...");
  currentDensity = PHRASE_LENGTH_MAX / 2;
  for (int v = 0; v< VOICES_MAX; v++) {
    for (int p = 0; p< PHRASE_LENGTH_MAX; p++) {
      //sequence[v][p] = random(NOTES_IN_MOOD); // populate sequence with random note pointers
      if(v == 0){
        sequence[v][p] = p % NOTES_IN_MOOD;
      }
      else{
        sequence[v][p] = random(NOTES_IN_MOOD); // populate sequence with random note pointers
      }
      if(v == 0){
        if(p%2 == 0){
          velocity[v][p] = MIDI_MIN_NOTE_ON_VELOCITY; // + random(MIDI_NOTE_ON_RANGE/2);
        }
        else{
          velocity[v][p] = 0;
        }
      }
      else{
        if(random(2) == 0){
          velocity[v][p] = MIDI_MIN_NOTE_ON_VELOCITY; // + random(MIDI_NOTE_ON_RANGE/2);
        }
        else{
          velocity[v][p] = 0;
        }
      }
    }
  }
  Serial.println("done.");
}

void makeSureAtLeastOneVoiceIsPlaying(){
  if(getNumberOfVoicesPlaying() < 1){
    voicesPlaying[random(VOICES_MAX)] = 1;
  }
}
int getNumberOfVoicesPlaying(){
  int currentVoicesPlaying = 0;
  for(int i=0; i<VOICES_MAX; i++){
    if(voicesPlaying[i] == 1){
      currentVoicesPlaying++;
    }
  }
  return currentVoicesPlaying;
}
void invertVoicesPlayingPlusBassline(){
  for(int i=0; i<VOICES_MAX; i++){
    if(voicesPlaying[i] == 1 && i != 2 && i != 3){
      voicesPlaying[i] = 0;
    }
    else{
      voicesPlaying[i] = 1;
    }
  }
}
void setBasslineDurrationToWalk(){
  noteDurations[2] = 4;
  noteDurations[3] = 8;
  noteDurations[7] = 4;
}
void setModeUpTwoOnHalf(){
  for(int i=0; i<VOICES_MAX; i++){
    if(random(2) == 1){
      scaleModes[i] = scaleModes[i]+2;
      if(scaleModes[i] >= NOTES_IN_MOOD){
        scaleModes[i] = scaleModes[i]-NOTES_IN_MOOD;
      }
    }
  }
}

void updateSequence(){
  if(allowEvolve == 0){
    return;
  }
  for (int v = 0; v < VOICES_MAX ; v++) {
    int densityStore = 0;
    // compute density of phrase.
    for (int p = 0; p < PHRASE_LENGTH_MAX; p++) {
      if(velocity[v][p] > 0){
        densityStore++;
      }
    }
    if (densityStore < currentDensity) {
      // randomly remove a rest (generate a new note!).
      velocity[v][random(PHRASE_LENGTH_MAX)] = MIDI_MIN_NOTE_ON_VELOCITY + random(MIDI_NOTE_ON_RANGE);
    }
    if (densityStore > currentDensity) {
      // randomly add a rest.
      velocity[v][random(PHRASE_LENGTH_MAX)] = 0;
    } 
  }
}

void loadBicycleBuiltForTwo(){
  currentDensity =  16;
  tempo          = 180;
  currentMood    =   4; // major scale.
  // first, set all velocities to zero.
  for (int v = 0; v< VOICES_MAX; v++) {
    for (int p = 0; p< PHRASE_LENGTH_MAX; p++) {
      velocity[v][p] = 0;
    }
  }
  // Bass line!
  noteDurations[3] = 4; // all quarted notes.
  sequence[3][0]   = 7; // 4 measures.
  sequence[3][1]   = 4;
  sequence[3][2]   = 2;
  sequence[3][3]   = 0;
  sequence[3][4]   = 3; // 4 measures 
  sequence[3][5]   = 3;
  sequence[3][6]   = 0;
  sequence[3][7]   = 0;
  sequence[3][4]   = 3; // 4 measures 
  sequence[3][5]   = 3;
  sequence[3][6]   = 0;
  sequence[3][7]   = 0;
  sequence[3][8]   = 6; // 4 measures 
  sequence[3][9]   = 5;
  sequence[3][10]   = 7;
  sequence[3][11]   = 5;
  sequence[3][12]   = 1; // 4 measures 
  sequence[3][13]   = 1;
  sequence[3][14]   = 4;
  sequence[3][15]   = 4;
  // first part melody
  noteDurations[3] = 2; // all eighth notes.
  sequence[3][0]   = 7; // 2 measures.
  sequence[3][1]   = 4;
  sequence[3][2]   = 2;
  sequence[3][3]   = 0;
  for(int i=0; i<16; i++){
    velocity[0][i] = 80;
    velocity[3][i] = 80;
  }


}






